@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2023
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -61,6 +61,7 @@ from barman.utils import (
     get_backup_id_using_shortcut,
     get_log_levels,
     parse_log_level,
+    parse_target_tli,
 )
 from barman.xlog import check_archive_usable
 
@@ -750,11 +751,19 @@ def rebuild_xlogdb(args):
             "backup_id",
             completer=backup_completer,
             help="specifies the backup ID to restore",
+            nargs="?",
+            default=None,
         ),
         argument(
             "destination_directory",
             help="the directory where the new server is created",
         ),
+        argument(
+            "--staging-wal-directory",
+            help="a staging directory in the target host for WAL files when performing "
+            "PITR. If unspecified, it uses a `barman_wal` directory inside the "
+            "destination directory.",
+        ),
         argument(
             "--bwlimit",
             help="maximum transfer rate in kilobytes per second. "
@@ -908,9 +917,73 @@ def restore(args):
     """
     server = get_server(args)
 
-    # Retrieves the backup
-    backup_id = parse_backup_id(server, args)
-    if backup_id.status not in BackupInfo.STATUS_COPY_DONE:
+    # PostgreSQL supports multiple parameters to specify when the recovery
+    # process will end, and in that case the last entry in recovery
+    # configuration files will be used. See [1]
+    #
+    # Since the meaning of the target options is not dependent on the order
+    # of parameters, we decided to make the target options mutually exclusive.
+    #
+    # [1]: https://www.postgresql.org/docs/current/static/
+    #   recovery-target-settings.html
+
+    target_options = [
+        "target_time",
+        "target_xid",
+        "target_lsn",
+        "target_name",
+        "target_immediate",
+    ]
+
+    specified_target_options = [
+        option for option in target_options if getattr(args, option, None)
+    ]
+    if len(specified_target_options) > 1:
+        output.error("You cannot specify multiple targets for the recovery operation")
+        output.close_and_exit()
+
+    target_option = (
+        specified_target_options[0] if len(specified_target_options) == 1 else None
+    )
+    target_tli = None
+    backup_info = None
+    if args.backup_id is not None:
+        backup_info = parse_backup_id(server, args)
+    else:
+        target = getattr(args, target_option) if target_option else None
+        # "Parse" the string value to integer for target_tli if passed as a string
+        # ("current", "latest")
+        target_tli = parse_target_tli(
+            obj=server.backup_manager, target_tli=args.target_tli
+        )
+        #  Error out on recovery targets that are not allowed.
+        if target_option in {"target_immediate", "target_xid", "target_name"}:
+            output.error(
+                "For PITR without a backup_id, the only possible recovery targets "
+                "are target_time and target_lsn. '%s' recovery target is not "
+                "allowed without a backup_id." % target_option
+            )
+            output.close_and_exit()
+        # Search for a candidate backup based on recovery targets if "backup_id" is None
+        elif target_option is None:
+            if target_tli is not None:
+                backup_id = server.get_last_backup_id_from_target_tli(target_tli)
+            else:
+                backup_id = server.get_last_backup_id()
+        elif target_option == "target_time":
+            backup_id = server.get_closest_backup_id_from_target_time(
+                target, target_tli
+            )
+        elif target_option == "target_lsn":
+            backup_id = server.get_closest_backup_id_from_target_lsn(target, target_tli)
+        # If no candidate backup_id is found, error out.
+        if backup_id is None:
+            output.error("Cannot find any candidate backup for recovery.")
+            output.close_and_exit()
+
+        backup_info = server.get_backup(backup_id)
+
+    if backup_info.status not in BackupInfo.STATUS_COPY_DONE:
         output.error(
             "Cannot restore from backup '%s' of server '%s': "
             "backup status is not DONE",
@@ -921,7 +994,7 @@ def restore(args):
 
     # If the backup to be recovered is compressed then there are additional
     # checks to be carried out
-    if backup_id.compression is not None:
+    if backup_info.compression is not None:
         # Set the recovery staging path from the cli if it is set
         if args.recovery_staging_path is not None:
             try:
@@ -942,13 +1015,13 @@ def restore(args):
                 "argument.",
                 args.backup_id,
                 server.config.name,
-                backup_id.compression,
+                backup_info.compression,
             )
             output.close_and_exit()
 
     # If the backup to be recovered is incremental then there are additional
     # checks to be carried out
-    if backup_id.is_incremental:
+    if backup_info.is_incremental:
         # Set the local staging path from the cli if it is set
         if args.local_staging_path is not None:
             try:
@@ -989,9 +1062,9 @@ def restore(args):
 
     # validate the rules against the tablespace list
     valid_tablespaces = []
-    if backup_id.tablespaces:
+    if backup_info.tablespaces:
         valid_tablespaces = [
-            tablespace_data.name for tablespace_data in backup_id.tablespaces
+            tablespace_data.name for tablespace_data in backup_info.tablespaces
         ]
     for item in tablespaces:
         if item not in valid_tablespaces:
@@ -1031,30 +1104,6 @@ def restore(args):
     if hasattr(args, "bwlimit"):
         server.config.bandwidth_limit = args.bwlimit
 
-    # PostgreSQL supports multiple parameters to specify when the recovery
-    # process will end, and in that case the last entry in recovery
-    # configuration files will be used. See [1]
-    #
-    # Since the meaning of the target options is not dependent on the order
-    # of parameters, we decided to make the target options mutually exclusive.
-    #
-    # [1]: https://www.postgresql.org/docs/current/static/
-    #   recovery-target-settings.html
-
-    target_options = [
-        "target_time",
-        "target_xid",
-        "target_lsn",
-        "target_name",
-        "target_immediate",
-    ]
-    specified_target_options = len(
-        [option for option in target_options if getattr(args, option)]
-    )
-    if specified_target_options > 1:
-        output.error("You cannot specify multiple targets for the recovery operation")
-        output.close_and_exit()
-
     if hasattr(args, "network_compression"):
         if args.network_compression and args.remote_ssh_command is None:
             output.error(
@@ -1066,7 +1115,7 @@ def restore(args):
             output.close_and_exit()
         server.config.network_compression = args.network_compression
 
-    if backup_id.snapshots_info is not None:
+    if backup_info.snapshots_info is not None:
         missing_args = []
         if not args.snapshot_recovery_instance:
             missing_args.append("--snapshot-recovery-instance")
@@ -1074,7 +1123,7 @@ def restore(args):
             output.error(
                 "Backup %s is a snapshot backup and the following required arguments "
                 "have not been provided: %s",
-                backup_id.backup_id,
+                backup_info.backup_id,
                 ", ".join(missing_args),
             )
             output.close_and_exit()
@@ -1082,7 +1131,7 @@ def restore(args):
             output.error(
                 "Backup %s is a snapshot backup therefore tablespace relocation rules "
                 "cannot be used.",
-                backup_id.backup_id,
+                backup_info.backup_id,
             )
             output.close_and_exit()
         # Set the snapshot keyword arguments to be passed to the recovery executor
@@ -1109,7 +1158,7 @@ def restore(args):
             output.error(
                 "Backup %s is not a snapshot backup but the following snapshot "
                 "arguments have been used: %s",
-                backup_id.backup_id,
+                backup_info.backup_id,
                 ", ".join(unexpected_args),
             )
             output.close_and_exit()
@@ -1120,8 +1169,9 @@ def restore(args):
     with closing(server):
         try:
             server.recover(
-                backup_id,
+                backup_info,
                 args.destination_directory,
+                wal_dest=args.staging_wal_directory,
                 tablespaces=tablespaces,
                 target_tli=args.target_tli,
                 target_time=args.target_time,
